
import galfitools 


def printWelcome():


    print("GALFITools: a library for GALFIT")

    print("Version:",galfitools.__version__)

    url = "https://github.com/canorve/GALFITools"
    
    print("webpage: "+url+"\n")


